﻿using System.Diagnostics;
using System.Threading;

using Microsoft.UI.Xaml;

namespace TestProject.WinUI;

/// <summary>
/// 앱
/// </summary>
public partial class App : MauiWinUIApplication
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 뮤텍스
    /// </summary>
    private static Mutex _mutex = null;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - App()

    /// <summary>
    /// 생성자
    /// </summary>
    public App()
    {
        this.InitializeComponent();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 단일 인스턴스 여부 구하기 - IsSingleInstance()

    /// <summary>
    /// 단일 인스턴스 여부 구하기
    /// </summary>
    /// <returns>단일 인스턴스 여부</returns>
    private static bool IsSingleInstance()
    {
        const string applicationID = "AF57FC2A-CAEF-466F-8D15-651A418258BE";

        _mutex = new Mutex(false, applicationID);

        GC.KeepAlive(_mutex);

        try
        {
            return _mutex.WaitOne(0, false);
        }
        catch(AbandonedMutexException)
        {
            _mutex.ReleaseMutex();

            return _mutex.WaitOne(0, false);
        }
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Protected

    #region MAUI 앱 생성하기 - CreateMauiApp()

    /// <summary>
    /// MAUI 앱 생성하기
    /// </summary>
    /// <returns>MAUI 앱</returns>
    protected override MauiApp CreateMauiApp() => MauiProgram.CreateMauiApp();

    #endregion
    #region 시작시 처리하기 - OnLaunched(e)

    /// <summary>
    /// 시작시 처리하기
    /// </summary>
    /// <param name="e">이벤트 인자</param>
    protected override void OnLaunched(LaunchActivatedEventArgs e)
    {
        if(!IsSingleInstance())
        {
            Process.GetCurrentProcess().Kill();
        }
        else
        {
            base.OnLaunched(e);
        }
    }

    #endregion
}